'use strict';
// Initial script copied/adapted from link below (MIT License)
// https://github.com/georgewfraser/vscode-javac/blob/a2a6fda/lib/Main.ts
var VSCode = require('vscode');
var Path = require('path');
var FS = require('fs');
var PortFinder = require('portfinder');
var Net = require('net');
var ChildProcess = require('child_process');
var VSCode_languageclient_1 = require('VSCode-languageclient');
PortFinder.basePort = 55747;
// this method is called when your extension is activated
// your extension is activated the very first time the command is executed
function activate(context) {
    console.log('Ceylon: activating');
    var ceylonExecutablePath = findExecutable('ceylon');
    console.log("Ceylon: path " + ceylonExecutablePath);
    if (ceylonExecutablePath == null) {
        VSCode.window.showErrorMessage("Couldn't locate ceylon in ceylon.home, $CEYLON_HOME or $PATH");
        return;
    }
    // Options to control the language client
    var clientOptions = {
        // Register the server for ceylon documents
        documentSelector: ['ceylon'],
        synchronize: {
            // Synchronize the setting section 'ceylon' to the server
            configurationSection: 'ceylon',
            fileEvents: [
                VSCode.workspace.createFileSystemWatcher('**/.ceylon/config'),
                VSCode.workspace.createFileSystemWatcher('**/*.ceylon')
            ]
        }
    };
    function createServer() {
        return new Promise(function (resolve, reject) {
            PortFinder.getPort(function (err, port) {
                console.log("Ceylon: port " + port);
                // make sure we can run Ceylon, and check the version.
                ChildProcess.execFile(ceylonExecutablePath, ["--version"], null, (function (error, stdout, stderr) {
                    if (error) {
                        VSCode.window.showErrorMessage("JAVA_HOME not set? Unable to verify the Ceylon version " +
                            "installed at " + ceylonExecutablePath +
                            ": " + error.message);
                        return;
                    }
                    var version = stdout.trim().substring(15);
                    if (!version.startsWith("1.3.0")) {
                        VSCode.window.showErrorMessage("Unsupported Ceylon version " + version + ". " +
                            "This version of the Ceylon Language VSCode Plugin " +
                            "requires Ceylon 1.3.0 and may not function correctly.");
                    }
                }));
                // TODO Specify repository 'remote=aether:settings.xml' for the
                //      snapshot maven dependencies. Or, include the deps in a flat repo?
                var cwd = context.asAbsolutePath("nodir");
                var lib = context.asAbsolutePath("lib");
                var modules = context.asAbsolutePath("out/modules");
                var args = [
                    'run',
                    ("--cwd=" + cwd),
                    '--rep', lib,
                    '--rep', modules,
                    '--auto-export-maven-dependencies',
                    'com.vasileff.ceylon.vscode/0.0.1',
                    port.toString(),
                    VSCode.workspace
                        .getConfiguration("ceylon")
                        .get("serverLogPriority").toString()
                ];
                Net.createServer(function (socket) {
                    resolve({
                        reader: socket,
                        writer: socket
                    });
                }).listen(port, function () {
                    var options = { stdio: 'inherit', cwd: VSCode.workspace.rootPath };
                    console.log("Ceylon: starting server: " + args);
                    // Run the ceylon language server
                    ChildProcess.execFile(ceylonExecutablePath, args, options);
                });
            });
        });
    }
    // Create the language client and start the client.
    var client = new VSCode_languageclient_1.LanguageClient('Ceylon Langauge Client', createServer, clientOptions);
    var disposable = client.start();
    // Push the disposable to the context's subscriptions so that the
    // client can be deactivated on extension deactivation
    context.subscriptions.push(disposable);
}
exports.activate = activate;
function findExecutable(binname) {
    binname = correctBinname(binname);
    // First search ceylon.home configuration option
    var ceylonHome = VSCode.workspace.getConfiguration("ceylon").get("home");
    if (ceylonHome) {
        var binpath = Path.join(ceylonHome, 'bin', binname);
        if (FS.existsSync(binpath)) {
            return binpath;
        }
    }
    // Then search each CEYLON_HOME bin folder
    if (process.env['CEYLON_HOME']) {
        var workspaces = process.env['CEYLON_HOME'].split(Path.delimiter);
        for (var i = 0; i < workspaces.length; i++) {
            var binpath = Path.join(workspaces[i], 'bin', binname);
            if (FS.existsSync(binpath)) {
                return binpath;
            }
        }
    }
    // Then search PATH parts
    if (process.env['PATH']) {
        var pathparts = process.env['PATH'].split(Path.delimiter);
        for (var i = 0; i < pathparts.length; i++) {
            var binpath = Path.join(pathparts[i], binname);
            if (FS.existsSync(binpath)) {
                return binpath;
            }
        }
    }
    // Else return the binary name directly (this will likely always fail downstream)
    return null;
}
function correctBinname(binname) {
    if (process.platform === 'win32')
        return binname + '.bat';
    else
        return binname;
}
// this method is called when your extension is deactivated
function deactivate() {
    console.log("Ceylon: deactivating");
}
exports.deactivate = deactivate;
//# sourceMappingURL=extension.js.map